import {
  select,
  csv,
  line,
  format,
  scaleLinear,
  scaleTime,
  scaleOrdinal,
  schemeCategory10,
  extent,
  max,
  axisLeft,
  axisBottom,
  curveBasis
} from 'd3'

import { colorLegend } from './colorLegend'
import { label } from './label'

const svg = select('svg')

const margin = { top: 60, right: 120, bottom: 72, left: 90 }
const width = +svg.attr('width')
const height = +svg.attr('height')
const innerWidth = width - margin.left - margin.right
const innerHeight = height - margin.top - margin.bottom

let selectedYear = '2018'

const g = svg.append('g')
	.attr('transform', `translate(${margin.left},${margin.top})`)

const render = data => {
  const circleRadius = 10
  const title = 'New line chart'
  
  const xValue = d => d.year
  const xAxisLabel = 'Year'
  
  const bornValue = d => d.bornlive
  const diedValue = d => d.died
  const yAxisLabel = 'people x 1000'
  
  const colorScale = scaleOrdinal(schemeCategory10)
  	colorScale.domain(['died', 'bornlive'])
  
  const xScale = d3.scaleUtc()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice()
  
  const yScale = scaleLinear()
    .domain([0, max(data, diedValue)])
    .range([innerHeight, 0])
    .nice()

  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15)
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
  
  const yAxisG = g.append('g').call(yAxis)
  yAxisG.selectAll('.domain').remove()
  
  // Append Y Axis label
  label(yAxisG, {
    yAxisLabelText: yAxisLabel,
    x: -innerHeight /2,
    y: -60,
    transform: 'rotate(-90)'
  })
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`)
  
  xAxisG.select('.domain').remove()
  
  label(xAxisG, {
    yAxisLabelText: xAxisLabel,
    x: innerWidth /2,
    y: 65,
    transform: ''
  })
  
  const lineGenerator = yValueFn => line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValueFn(d)))
  	.curve(curveBasis)
  
  const pathGenerator = {
  	bornlive: lineGenerator(bornValue),
    died: lineGenerator(diedValue)
  }
  
  g.selectAll('.line-path-new').data(['bornlive', 'died'])
  	.enter().append('path')
    	.attr('class', 'line-path')
  		.attr('stroke', d => colorScale(d))
  		.attr('d', d => pathGenerator[d](data))
  
  svg.append('g')
    .attr('transform', 
          'translate(' + (innerWidth + 120) + ',270)')
    .call(colorLegend, {
      colorScale,
      circleRadius: 8,
      spacing: 40,
      textOffset: 14
    });
  
  const selectedYearDate = new Date(selectedYear)
  
  // Get current year data
  const currentYearData = data.filter(d => {
    return d.year.getFullYear() === parseInt(selectedYear)
  })[0]
  console.log( currentYearData.died, currentYearData.bornlive )
  console.log( max([currentYearData.died, currentYearData.bornlive]))
  /*
  // Selected year
  g.append('line')
  		.attr('x1', xScale(selectedYearDate))
  		.attr('x2', xScale(selectedYearDate))
  		.attr('y1', innerHeight)
  		.attr('y2', yScale( max([currentYearData.died,
                              currentYearData.bornlive]) ))
  		.attr('stroke', 'brown')
  */
  g.append('text')
      .attr('class', 'title')
      .attr('y', -30)
      .text(title)
}

d3.csv('https://vizhub.com/henrilappalainen/datasets/pop-finland.csv')
  .then(data => {
	  data.forEach(d => {
    d.population = +d.population / 1000
    d.year = new Date(d.year)
    d.died = +d.died / 1000
    d.bornlive = +d.bornlive / 1000
  })
  render(data);
})
